import { remove } from 'lodash'
import type { RefObject } from 'react'
import { loadASMAMicroAPP } from '../loadASMAMicroApp'
import type { Entry, MicroApp } from '../qiankun'

const LoaderQueue: IAppLoaderQueue = {}

function registerLoader(app_name: string, loader: ILoader) {
    if (!LoaderQueue[app_name]) {
        LoaderQueue[app_name] = []
    }

    LoaderQueue[app_name]!.push(loader)
}

interface IAppLoaderQueue {
    [app_names: string]: ILoader[]
}
interface ILoader {
    id: string
    init: () => MicroApp
}

export interface IMfComponentLoader<T> extends Pick<React.HTMLAttributes<HTMLDivElement>, 'className'> {
    app: { name: string; entry: Entry }
    props: IMicroAppProps<T>
    placeholder?: string
}

export type IMicroAppProps<T> = { component_path: string } & T
const areLoadersInProcess = {} as Record<string, boolean>

async function resolveLoaders(app_name: string) {
    areLoadersInProcess[app_name] = true

    while (areLoadersInProcess[app_name]) {
        const loader_to_resolve = LoaderQueue[app_name]?.[0]

        if (loader_to_resolve) {
            await loader_to_resolve.init().bootstrapPromise

            remove(LoaderQueue[app_name] || [], (l) => l.id === loader_to_resolve.id)
        }

        if (!LoaderQueue[app_name]?.length) {
            areLoadersInProcess[app_name] = false
        }
    }
}

export function initLoadMicroApp(
    app: { name: string; entry: Entry },
    props: IMicroAppProps<{}>,
    containerRef: RefObject<HTMLDivElement>,
    setLoadedApp: (lApp: MicroApp) => void,
) {
    function init() {
        const loadedapp = loadASMAMicroAPP({
            name: app.name,
            entry: app.entry,
            container: containerRef.current!,
            props,
        })

        setLoadedApp(loadedapp)

        return loadedapp
    }

    registerLoader(app.name, { id: props.component_path, init })

    if (!areLoadersInProcess[app.name]) {
        resolveLoaders(app.name)
    }
}
